
#include "genlevel.h"
#include <math.h>

//#define RAND_LEVEL_DEBUG
	#include <iostream>

/* 
	 prints out the level generated by the random level generator
	 */
ostream & operator<<(ostream & os, RandomLevelGenerator & rlg){
	for (int k = 0; k <  rlg.getFloors( ) ; k++){
		for (int i = 0; i <  rlg.getWidth( ) ; i++){
			for (int j = 0; j <  rlg.getHeight( ) ; j++){
				os << rlg.getValue(i,j,k);
			}
			os << endl;
		}
		os << endl << "Floor # " << k << endl;
	}
	return os;
}

/*
	This is the big, ugly method that actually generates the
	random level.
	 */
void RandomLevelGenerator::generate_level (){
	#ifdef RAND_LEVEL_DEBUG
	cerr << "Initializing random level..  " ;
	#endif

	// initialize the level to all walls
	for (int k = 0; k <  getFloors( ) ; k++)
		for (int i = 0; i <  getWidth( ) ; i++)
			for (int j = 0; j <  getHeight( ) ; j++){
				the_level[getIndex(i,j,k)] = k_WALL;
			}
	
	#ifdef RAND_LEVEL_DEBUG
	cerr << "done." << endl;
	cerr << "Setting the maximum line length.. " ;
	#endif
	// choose the maximum line length between 5 and k_LINELENGTH
	int maxlinelength = 7 + (rand() % (k_LINELENGTH - 5));

	#ifdef RAND_LEVEL_DEBUG
	cerr << "  max line length is " << maxlinelength << endl;
	cerr << "  done." << endl;
	cerr << "Choosing starting point.." ;
	#endif
	
// The following code will choose a starting point in the array, and
// randomly carve legal paths leading from it.

	the_level [getIndex(2 + (rand() % ( getWidth( )  - 4)) , 2+ (rand() % ( getHeight( )  - 4)) , 0)] = k_HALLWAY;
	
	#ifdef RAND_LEVEL_DEBUG
	cerr << "done." << endl;
	cerr << "Generating level.." << endl ;
	#endif
	
	// loop through the floors in the array. if we're on floor zero,
	// choose a random starting point for paths to extend from. otherwise
	// choose a random starting point such that the floor below the current
	// floor has a hallway there to ensure that we can connect the floors
	// at some point

	int numits; // the number of iterations.
	int numhalls; // the number of hallway spaces in the current floor

	for (int k = 0; k <  getFloors( ) ; k++){
		if ( k != 0){
			#ifdef RAND_LEVEL_DEBUG
			cerr << "|" << endl ;
			cerr << "  Choosing a starting point for floor " << k << ".." << endl ;
			#endif
	
			int a, b;
			do {
				a = 2 + (rand() % ( getWidth( )  - 4));
				b = 2+ (rand() % ( getHeight( )  - 4));
			}while (	the_level [ getIndex ( a , b , k-1 ) ] != k_HALLWAY);
			the_level[ getIndex ( a , b , k ) ] = k_HALLWAY;
		}
		#ifdef RAND_LEVEL_DEBUG
		cerr << "  Generating maze for floor " << k << ".." << endl ;
		cerr << "  Iterations: |" ;
		#endif
		
		numits = 0;
		numhalls = 0;
		while (numits < k_NUMITS && ((100 * numhalls) / (getHeight( ) * getWidth( ) )) < 40){
			#ifdef RAND_LEVEL_DEBUG
			cerr << ( (numits % 10 == 9) ? '+' : '-') ;
			#endif
			for (int i = 0; i <  getWidth( ) ; i++){
				for (int j = 0; j <  getHeight( ) ; j++){
					// first check to see that this is a hallway element. we only
					// want to extend paths from existing paths
					if (the_level[ getIndex ( i , j , k ) ] == k_HALLWAY){
						// there are four directions we could possibly move: up
						// down, right, or left. pick one randomly.
						
						// although there are only four options, i apparently have a poor-quality
						// random number generator with a simple pattern to the remainders mod 4
						int dir = rand() % 7;
					
						// Now, if the direction that we chose is a good one, keep carving walls
						// in that direction until one of the following occurs:
						//       (a)  we hit an existing wall
						//       (b)  the next square in this direction cannot be a 
						//            wall because squares within two units of the next
						//            square (in the directions perpendicular to the
						//            direction we're moving) are not walls
						//       (c)  we exceed the maximum line length
						//       (d)  the next square is an edge. hallways on the edge
						//            must match up with hallways on the opposite edge, so
						//            we'll take care of edge hallways later
						if (dir == k_UP && the_level[ getIndex ( i , j-1 , k ) ] == k_WALL){
							int a = 1;
							while ( (the_level[ getIndex ( i-1 , j-a , k ) ] == k_WALL) && (the_level[ getIndex ( i-2 , j-a , k ) ]  == k_WALL) && (the_level[ getIndex ( i+1 , j-a , k ) ] == k_WALL) && (the_level[ getIndex ( i+2 , j-a , k ) ] == k_WALL) && (a < maxlinelength) && (j-a >= 2)){
								numhalls++;
								the_level[ getIndex ( i , j-a , k ) ] = k_HALLWAY;
								a++;
							}

						} else if (dir == k_DOWN && the_level[ getIndex ( i , j+1 , k ) ] == k_WALL){
							int a = 1;
							while ( (the_level[ getIndex ( i-1 , j+a , k ) ] == k_WALL) && (the_level[ getIndex ( i-2 , j+a , k ) ] == k_WALL) && (the_level[ getIndex ( i+1 , j+a , k ) ] == k_WALL) && (the_level[ getIndex ( i+2 , j+a , k ) ] == k_WALL) && (a < maxlinelength) && (j+a <  getHeight( )  - 2)){
								numhalls++;
								the_level[ getIndex ( i , j+a , k ) ] = k_HALLWAY;
								a++;
							}

						} else if (dir == k_RIGHT && the_level[ getIndex ( i+1 , j , k ) ] == k_WALL){
							int a = 1;
							while ( (the_level[ getIndex ( i+a , j-1 , k ) ] == k_WALL) && (the_level[ getIndex ( i+a , j-2 , k ) ] == k_WALL) && (the_level[ getIndex ( i+a , j+1 , k ) ] == k_WALL) && (the_level[ getIndex ( i+a , j+2 , k ) ] == k_WALL) && (a < maxlinelength) && (i+a <  getWidth( )  - 2)){
								numhalls++;
								the_level[ getIndex ( i+a , j , k ) ] = k_HALLWAY;
								a++;
							}

						} else if (dir == k_LEFT && the_level[ getIndex ( i-1 , j , k ) ] == k_WALL){
							int a = 1;
							while ( (the_level[ getIndex ( i-a , j-1 , k ) ] == k_WALL) && (the_level[ getIndex (  i-a , j-2 , k ) ] == k_WALL) && (the_level[ getIndex ( i-a , j+1 , k ) ] == k_WALL) && (the_level[ getIndex ( i-a , j+2 , k ) ] == k_WALL) && (a < maxlinelength) && (i-a >= 2)){
								numhalls++;
								the_level[ getIndex ( i-a , j , k ) ] = k_HALLWAY;
								a++;
							}
						}
					}					
				}
			}
			numits++;
		}
	}

	// if there are any "adjacent corners" like the following:
	//
	//	   .
	//     .
	//   ...
	//      ....
	//      .
	//
	// delete one of them.

	{
		int numbadcorners = 0;
		do {
			numbadcorners = 0;
	for (int k = 0; k <  getFloors( ) ; k++){
		for (int j = 1; j <  getHeight( ) -1; j++){
			for (int i = 1; i <  getWidth( ) -1; i++){
				if (the_level[getIndex(i-1, j-1, k)] != k_WALL
						&& the_level[getIndex(i, j-1, k)] == k_WALL
						&& the_level[getIndex(i-1, j, k)] == k_WALL
						&& the_level[getIndex(i, j, k)] != k_WALL
						){
					numbadcorners++;
						if (rand() > rand()){
							the_level[getIndex(i-1, j-1, k)] = k_WALL;
						} else {
							the_level[getIndex(i, j, k)] = k_WALL;
						}
				} else if (the_level[getIndex(i-1, j-1, k)] == k_WALL
						&& the_level[getIndex(i, j-1, k)] != k_WALL
						&& the_level[getIndex(i-1, j, k)] != k_WALL
						&& the_level[getIndex(i, j, k)] == k_WALL
						){
					numbadcorners++;
						if (rand() > rand()){
							the_level[getIndex(i, j-1, k)] = k_WALL;
						} else {
							the_level[getIndex(i-1, j, k)] = k_WALL;
						}
				}
			}
		}
	}
		} while (numbadcorners != 0);
	}
		



	// remove any spaces with <2 adjacent spaces to eliminate dead ends.
	int numadj; // the number of adjacent hallway squares to the current square
	int numchanged; // the number of squares changed in the most recent iteration
	do {
		numchanged = 0; // initialize numchanged for this iteration
		
		for (int k = 0; k <  getFloors( ) ; k++){
			for (int j = 1; j <  getHeight( ) -1; j++){
				for (int i = 1; i <  getWidth( ) -1; i++){
					if (the_level[ getIndex ( i , j , k ) ] == k_HALLWAY){
						numadj = 0;// initialize numadj for this space
						
						// check the four adjacent squares and increment numadj
						// for each adjacent square that is a hallway
						numadj += (the_level[ getIndex ( i , j+1 , k ) ]!=k_WALL ? 1 : 0);
						numadj += (the_level[ getIndex ( i , j-1 , k ) ]!=k_WALL ? 1 : 0);
						numadj += (the_level[ getIndex ( i+1 , j , k ) ]!=k_WALL ? 1 : 0);
						numadj += (the_level[ getIndex ( i-1 , j , k ) ]!=k_WALL ? 1 : 0);
						
						// if a space is adjacent to only one square, then it is the
						// last space in a blind alley, so delete it. we want everything
						// nice and connected.
						if (numadj <= 1){
							numchanged ++;
							the_level[ getIndex ( i, j, k ) ] = k_WALL;
						}
					}
				}
			}
		}
	#ifdef RAND_LEVEL_DEBUG
	cerr << "    " << numchanged << " spaces removed in this iteration." << endl ;
	#endif
	} while (numchanged > 0);

	#ifdef RAND_LEVEL_DEBUG
	cerr << "    done." << endl ;
	cerr << "  Adding stairs up and down.." << endl ;
	#endif
	
	
	// now, we want to add the stairs between levels. we'll randomly choose squares and check
	// that we've chosen hallway squares in each level.
	for (int k = 1; k < getFloors(); k++){
		int a, b;
		int numtries = 0;
		int realnumjumps = 0;
		#ifdef RAND_LEVEL_DEBUG
		cerr << "    Trying to add stairs between floor " << (k-1) << " and " << k << ".." << endl ;
		#endif
		for (int num_jumps = 0; num_jumps < .5*sqrt(getWidth()*getHeight()) || ( realnumjumps == 0) ; num_jumps++){
				a = 2 + (rand() % ( getWidth( )  - 4));
				b = 2+ (rand() % ( getHeight( )  - 4));
			if (numtries < 20 && isHallway(a, b, k) && isHallway(a, b, k-1)){
				#ifdef RAND_LEVEL_DEBUG
				cerr << "      Added jump at ( " << a << " , " << b << " ). num_jumps=" << num_jumps << ". realnumjumps="  << realnumjumps << endl ;
				#endif
				the_level[ getIndex ( a , b , k ) ] = k_DROPDOWN;
				the_level[ getIndex ( a , b , k-1 ) ] = k_JUMPUP;
				realnumjumps++;
			}
		}
	}

	#ifdef RAND_LEVEL_DEBUG
	cerr << "    done." << endl ;
	cerr << "  Adding vertical edge hallways.." << endl ;
	#endif
	// add edge hallways that wrap around the level. we want to try to add one 
	// for every 6 edge squares. We're doing this randomly, so it's possible
	// that none or very few attempts will succeed.
	// 
	// first, let's add the ones along the vertical edges..
	for (int k = 0; k <  getFloors( ) ; k++){
		for (int a = 0; a < 1+( getHeight( ) /10); a++){
			// pick a random y-value
			int y_val = 4 + (rand() % ( getHeight( ) -8));

			// start at x = 0 and move right until you hit a hallway, or until
			// we get to a point where carving a new hallway would invalidate
			// the level
			int x_val_left = 0;
			while (the_level[ getIndex ( x_val_left , y_val , k ) ] == k_WALL && the_level[ getIndex ( x_val_left , y_val-1 , k ) ] == k_WALL && the_level[ getIndex ( x_val_left , y_val+1 , k ) ] == k_WALL && the_level[ getIndex ( x_val_left , y_val-2 , k ) ] == k_WALL && the_level[ getIndex ( x_val_left , y_val+2 , k ) ] == k_WALL && x_val_left != getWidth()-1){
				x_val_left++;
			}
			
			// do the same thing but from the rightmost edge inward
			int x_val_right =  getWidth( ) -1;
			while (the_level[ getIndex ( x_val_right , y_val , k ) ] == k_WALL && the_level[ getIndex ( x_val_right , y_val-1 , k ) ] == k_WALL && the_level[ getIndex ( x_val_right , y_val+1 , k ) ] == k_WALL && the_level[ getIndex ( x_val_right , y_val-2 , k ) ] == k_WALL && the_level[ getIndex ( x_val_right , y_val+2 , k ) ] == k_WALL && x_val_right != 0){
				x_val_right--;
			}
			
			// now, if both those while() loops terminated with x-values that are hallways, we
			// found a good spot! initialize all these spaces to be edge hallways
			if (the_level[ getIndex ( x_val_right , y_val , k ) ] == k_HALLWAY && the_level[ getIndex ( x_val_left , y_val , k ) ] == k_HALLWAY){
				#ifdef RAND_LEVEL_DEBUG
				cerr << "    Added edge hallway at y=" << y_val << endl ;
				#endif

				for (int b = 0; b< x_val_left; b++)
					the_level[ getIndex ( b , y_val , k ) ] = k_EDGEHALL;
				for (int b =  getWidth( ) -1; b> x_val_right; b--)
					the_level[ getIndex ( b , y_val , k ) ] = k_EDGEHALL;
			}
		}
	}

	#ifdef RAND_LEVEL_DEBUG
	cerr << "    done." << endl ;
	cerr << "  Adding horizontal edge hallways.." << endl ;
	#endif
	// and now, let's add the ones along the horizontal edges..
	for (int k = 0; k <  getFloors( ) ; k++){
		for (int a = 0; a < 1+( getWidth( ) /10); a++){
			// pick a random x-value
			int x_val = 4 + (rand() % ( getWidth( ) -8));
			
			
			// start at y = 0 and move down until you hit a hallway, or until
			// we get to a point where carving a new hallway would invalidate
			// the level
			int y_val_left = 0;
			while (the_level[ getIndex ( x_val , y_val_left , k ) ] == k_WALL && the_level[ getIndex ( x_val+1 , y_val_left , k ) ] == k_WALL && the_level[ getIndex ( x_val-1 , y_val_left , k ) ] == k_WALL && the_level[ getIndex ( x_val+2 , y_val_left , k ) ] == k_WALL && the_level[ getIndex ( x_val-2 , y_val_left , k ) ] == k_WALL && y_val_left != getHeight()-1){
				y_val_left++;
			}

			// do the same thing but from the bottommost edge upward
			int y_val_right =  getHeight( ) -1;
			while (the_level[ getIndex ( x_val , y_val_right , k ) ] == k_WALL && the_level[ getIndex ( x_val+2 , y_val_right , k ) ] == k_WALL && the_level[ getIndex ( x_val-1 , y_val_right , k ) ] == k_WALL && the_level[ getIndex ( x_val-2 , y_val_right , k ) ] == k_WALL && the_level[ getIndex ( x_val+1 , y_val_right , k ) ] == k_WALL && y_val_right != 0){
				y_val_right--;
			}
			
			// now, if both those while() loops terminated with y-values that are hallways, we
			// found a good spot! initialize all these spaces to be edge hallways
			if (the_level[ getIndex ( x_val , y_val_right , k ) ] == k_HALLWAY && the_level[ getIndex ( x_val , y_val_left , k ) ] == k_HALLWAY){
				#ifdef RAND_LEVEL_DEBUG
				cerr << "    Added edge hallway at x=" << x_val << endl ;
				#endif

				for (int b = 0; b< y_val_left; b++)
					the_level[ getIndex ( x_val , b , k ) ] = k_EDGEHALL;
				for (int b =  getHeight( ) -1; b> y_val_right; b--)
					the_level[ getIndex ( x_val , b , k ) ] = k_EDGEHALL;
			}
		}
	}

	#ifdef RAND_LEVEL_DEBUG
	cerr << "    done." << endl ;
	cerr << "  Placing pellets.." ;
	#endif
	for (int k = 0; k <  getFloors( ) ; k++)
		for (int i = 0; i <  getWidth( ) ; i++)
			for (int j = 0; j <  getHeight( ) ; j++)
				if (isHallway(i, j, k)){
						// we'll give it a 1 in 30 chance that a pellet is a power
						// pill rather than a regular pellet
					if (rand() % 30 == 1)
						the_level[getIndex(i,j,k)] = k_POWER_PILL;
					else
						the_level[getIndex(i,j,k)] = k_HALLWAY_WITH_PELLET;
				}

	// now we'll pick random spaces in the level as our pacman and
	// ghost respawn points
	{
		// temporary variables used to denote the x, y, and z values 
		// (repectively) of the randomly chosen space that we're trying
		// to make into a respawn point.
		int i, j, k;

		while ( maxpacmen > 0 || maxghosts > 0){
			do{
				i = rand () % getWidth();
				j = rand () % getHeight();
				k = rand() % getFloors();
			}while (! isHallway (i, j, k));

			if (maxpacmen > 1 ){
				// if there is more than one pacman spawn point,
				// then set this hallway space to be a "regular"
				// pacman spawn point.
				the_level[getIndex(i,j,k)] = k_PACMAN_RESPAWN;
				maxpacmen --;
				
			} else if ( maxpacmen == 1 ){
				// if we need only one more pacman spawn point, 
				// set this last one to be the main pacman
				// spawn point
				the_level[getIndex(i,j,k)] = k_MAIN_PACMAN_RESPAWN;
				maxpacmen = 0;

			} else if (maxghosts > 0){
				// set this point to be a ghost spawn point
				the_level[getIndex(i,j,k)] = k_GHOST_RESPAWN;
				maxghosts --;
			}
		}

	}

	// and finally, we add item spaces where items may randomly appear on
	// the board.
	{
		// temporary variables used to denote the x, y, and z values 
		// (repectively) of the randomly chosen space that we're trying
		// to make into a respawn point.
		int i, j, k;


		while (rand () % ((getHeight() * getWidth() * getFloors() )/80) != 0){
			do{
				i = rand () % getWidth();
				j = rand () % getHeight();
				k = rand() % getFloors();
			}while (! isHallway (i, j, k));
			the_level[getIndex(i, j, k)] = k_ITEM;
		}
	}
	
	#ifdef RAND_LEVEL_DEBUG
	cerr << "done." << endl ;
	#endif
}

int RandomLevelGenerator::getIndex(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return 0;
	return (width * height * k) + (j * width) + i;
}

char RandomLevelGenerator::getValue(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return 0;
	return the_level[getIndex(i, j, k)];
}


// ***********************************************************
// boolean accessors for space type..
//
bool RandomLevelGenerator::isHallway(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return false;
	return (getValue(i, j, k) == k_HALLWAY || getValue(i, j, k) == k_HALLWAY_WITH_PELLET);
}

bool RandomLevelGenerator::isWall(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return true;
	return (getValue(i, j, k) == k_WALL);
}

bool RandomLevelGenerator::isEdgehall(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return false;
	return (getValue(i, j, k) == k_EDGEHALL);
}

bool RandomLevelGenerator::isJumpUp(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return false;
	return (getValue(i, j, k) == k_JUMPUP);
}

bool RandomLevelGenerator::isDropDown(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return false;
	return (getValue(i, j, k) == k_DROPDOWN);
}

bool RandomLevelGenerator::isPassable(int i, int j, int k){
	if (i >= width || j >= height || k >= floors)
		return false;
	return (! isWall(i, j, k));
}



// ***********************************************************
// accessors for dimensions..
//
int RandomLevelGenerator::getWidth(void){
	return width;
}

int RandomLevelGenerator::getHeight(void){
	return height;
}

int RandomLevelGenerator::getFloors(void){
	return floors;
}


// constructors
RandomLevelGenerator::RandomLevelGenerator (int w, int h, int f, int p, int g){
	srand (time (NULL));
	
	width = w; 
	height = h;
	floors = f;

	maxpacmen = p;
	maxghosts = g;
	
	the_level = new char[w * h * f];
	generate_level();
}

// destructor
RandomLevelGenerator::~RandomLevelGenerator( void ){
	delete (the_level);
}
