/* Revision History

	4/06
	Sleep function
		changed the Sleep(33) functions
		it doesn't compile under linux 
		substituted
			struct timespec to_wait;
			to_wait.tv_sec = 0;
			to_wait.tv_nsec = 33000000; //33 million nanoseconds = 33 milliseconds
			nanosleep(&to_wait,NULL);
		instead of using glTimeDelay, or whatever the OpenGL one is

	4/14
	Chat Window
		added chat window stuff to this main loop.
		Everything, especially player list, still needs to be included
		in the GameBoard class.  Hopefully that won't be too bad.

	Zoom Level
		added the zoom level using CTRL-A and CTRL-Z
		I can't believe that is supported by OpenGL, I certainly didn't find it
		in any of the stupid OpenGL books that I have :/  it's done using a call
		to glutGetModifiers

	4/15 - 
	Platform Independence
		added some define statements that should take care of the Sleep function
		problem across windows and linux, did the same thing for some includes
		for time and sys/time.h

	4/17 -
	Includes
		changed some include files, they didn't work in linux.

	4/27
	Removing Sprites
		the graphics were having TERRIBLE FPS on linux machines, they work great on mine,
		but everything points to one problem -> textures.
		It's possible that the lack of 3d acceleration is killing the FPS because of the
		rapid swaping of textures, because the player drawing functions don't really do anything
		oh well, back to the drawing board

	4/27
	Removing Smoothing
		3 little lines were killing the performance on linux machines in UGCS, they're gone now

	New Graphics
		Got rid of sprites because textures just kill performance on non-hardware
		accelerated machines.  I put in some generic polygons for drawing the ghost
		they're not very good right now, I'll be working on that later though becase
		right now they get the job done.

	5/4
	Removing Depth Testing
		Another time killer is gone, depth testing!!  I had removed depth testing from all of the
		drawing methods, but I hadn't turned it off :/  that's gone now, glTranslatef was changed
		to glScalef and the gluPerspective was changed to a more presentable glOrtho call.
		It seems to be going much much faster now.

  	5/13
	Added Level Drawing
		Added the code for the drawing of a level.  It's too bloated, and there's a lot of
		repetition, but it works, and I didn't want to concentrate on condensing it right now
		especially because in it's current form, it's optimized.  I will be integrating the
		code into the level class when it's ready.

	6/05
	Added ServerLoop and Collision function
		Due to lack of backbone support, we have had to move the mechanics of the game
		into the opengl.cpp file.  The game mechanics are now controlled by a large
		function called ServerLoop.  ServerLoop should have been done in a Server object
		of some kind.
*/

//TODO:
//add a remove texture function
//add semaphores to limit accessing of the playerlist...?
//add extrapolation to multiplayer, see ServerLoop for an explanation

#include <iostream>
#include <string>
#include <map>
#ifndef WIN32
#include <sys/time.h>
#endif

#include "image.h"
#include "text.h"
#include "opengl.h"
#include "playerlist.h"
#include "chatwnd.h"
#include "sidebar.h"
#include "TCP.h"
#include "RUDP.h"
#include "Level.h"

#ifdef _MENU_TEST_MODE_
#include "menupane.h"
#endif

//playerlist is global now, but it should be GameBoard
//or Gameclient when this thing is done.
//I just need something to draw right now to test the player objects
extern PlayerList playerlist;
extern TCP* tcp;
extern RUDP* rudp;
extern ChatWnd chatwnd;

//set this on or off if you want to compile opengl.cpp in singleplayer or network play mode
#define SINGLEPLAYER

//don't need this stuff if we're in single play mode
#ifndef SINGLEPLAYER
extern UDPacket* outpacket;
extern long ms;
extern pthread_mutex_t graphicsMutex;
#endif

extern Level * testlevel;

#ifdef _MENU_TEST_MODE_
	extern MenuPane * mp;
#endif

using namespace std;
//temp level stuff

//this is global now, it should be moved with playerlist into the GameBoard object...
//but we don't have one, so it will have to stay global
SideBar sidebar(&playerlist);
extern int myID;

//counter used when ghosts turn edible
int ediblecount = 0;
bool edible = false;


//this is global now, it should be moved with playerlist into the GameBoard object...
GLfloat zoomlevel = .4;


//holder function for server operations.  The Clients just receive data from the server
//so they don't need to do this kind of checking
void ServerLoop();
void Collision(Player * tempplayer, Player * otherguy);


//the main rendering function for opengl.
//this function should be kept as sparse as possible, because ALL
//the drawing should be done in the individual draw methods of all the nouns
//or objects on screen
void RenderScene()
{

	#ifndef SINGLEPLAYER
	pthread_mutex_lock(&graphicsMutex);
	#endif

	//sleep to synchronize the clients and server...this should
	//take much more than the processing time of the rendering
	//
	// NOTE: this function seems to only work in windows. i'm 
	// commenting it out to test the graphics code.
	
#ifdef WIN32
	Sleep(33);
#endif

/*
#ifndef WIN32
#ifndef SINGLEPLAYER

	struct timespec to_wait;
	to_wait.tv_sec = 0;
	to_wait.tv_nsec = 33000000; //33 million nanoseconds = 33 milliseconds
	nanosleep(&to_wait,NULL);
#endif
#endif
*/

	//clear the system
	glClear(GL_COLOR_BUFFER_BIT);


	//activate the model view matrix, change this to the ortho matrix to render
	//the sidebar and chat window
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	//save the base state
	glPushMatrix();


	glTranslatef(-10, 0, 0);//move out to get a better view

	glScalef(zoomlevel, zoomlevel, 1);
	glColor4f(1, 1, 1, 1);


//	move the entire system to the place where this player's character is
//	the upper left space is at 0, 0.  I think, and the system goes
//	down and right from there in increments detailed in the draw level function
//	use that to determine where they go based on what space they're in, and their
//	fractional distance...
//	UPDATE: not using fracdist anymore because it's not ready, just using X, Y coords

	glTranslatef(-playerlist.getPlayer(myID)->getX(), -playerlist.getPlayer(myID)->getY(), 0); 



	//here we go
	//draw the level first because it has to lie under everything else
	//the level will call the draw methods of the spaces in it after it
	//draws itself
	testlevel->draw();

	//then draw the players
	playerlist.draw();
	
#ifndef SINGLEPLAYER
	//only draw the chat window if we're playing in multiplayer mode
	chatwnd.draw();
#endif
	//draw the sidebar over the chatwindow
	sidebar.draw();


	glPopMatrix();

	glutSwapBuffers();

	//if we are in single player mode, perform the game mechanics here
	#ifdef SINGLEPLAYER
	ServerLoop();
	#endif

	#ifndef SINGLEPLAYER
	pthread_mutex_unlock(&graphicsMutex);
	#endif
}

//this function is used for a collision with a ghost!
//some one is going to get eaten in here, this will take care of it
//this function is duplicated on the console_server and was written to
//be easily ported to it
void Collision(Player * tempplayer, Player * otherguy)
{

	//if the tempplayer is a pacman and the otherguy is a ghost
	if(!tempplayer->isAGhost() && otherguy->isAGhost())
	{	

		//make a temp ghost*
		Ghost * tempghost = (Ghost*)otherguy;

		//make a temp pacman*
		Pacman * temppacman = (Pacman*)tempplayer;

		//if the ghost is not dead
		if(!tempghost->isDead())
		{
			//eat the ghost if it is edible, or else kill the pacman
			if(tempghost->isEdible() && !tempghost->isDead())
			{
				//kill the ghost
			//	cout<<"eat the ghost first"<<endl;
				
				//award points too
				temppacman->changeScore(1000);
				tempghost->setDead(150);
			}
			//if pacman isn't already dead, eat him here
			else if(!temppacman->isDead())
			{
				//kill the pacman
				//cout<<"kill the pacman second"<<endl;
				
				//award points too
				tempghost->changeScore(500);
				temppacman->setDead();
			}
		}
		
	}

	//if the tempplayer is a ghost and the otherguy is a pacman
	else if(tempplayer->isAGhost() && !otherguy->isAGhost())
	{
		//make a temp ghost*
		Ghost * tempghost = (Ghost*)tempplayer;
		
		//make a temp pacman*
		Pacman * temppacman = (Pacman*)otherguy;
		

		//if the ghost is not dead
		if(!tempghost->isDead())
		{
			//eat the pacman if the ghost is not edible, or else kill the ghost
			if(tempghost->isEdible() && !tempghost->isDead())
			{
				//kill the ghost
				//cout<<"eat the ghost third, he's edible"<<endl;

				//award points too
				otherguy->changeScore(1000);
				tempghost->setDead(150);
			}
			//if pacman isn't already dead, eat him here
			else if(!temppacman->isDead())
			{
				//kill the pacman
				//cout<<"kill the pacman forth"<<endl;
				
				//award points too
				tempghost->changeScore(100);
				temppacman->setDead();
			}
		}
	}
}


//the ServerLoop function handles all of the game mechanics, item eating, collisions
//what do to when you hit a wall and so on.  If the game mechanics need to be changed
//it should be done in this function.  It was written to be easily ported over
//to the console_server 
//currently there is no extrapolation for network play.  In order to put extrapolation in
//and to make the game run a hell of a lot smoother, ServerLoop needs to be slightly altered
//so that it doesn't pick random directions for all characters, and it doesn't
//make any game altering changes, like eating pacmen or ghosts or eating what spaces contain
//  That way, the game will progress as normal and the server can send updates every half a
//second or so, that seems like a reasonable refresh time for Multiplayer Pacman 
void ServerLoop()
{
	//cout<<"begin server loop"<<endl;
	if(edible)
	{
		ediblecount++;
		if(ediblecount == 75)
		{
			playerlist.resetGhostsEdible();	
			ediblecount = 0;
			edible = false;
		}
	}

	//loop through all of the players each time
        map <int, Player*>::iterator i;   
        map <int, Player*> * tempplayermap = playerlist.getPlayerMap();
        for(i = tempplayermap->begin(); i != tempplayermap->end(); i++)
        {
                Player * tempplayer;
                tempplayer = i->second;



		//temp stuff for emulating other players
		if(tempplayer->getPlayerID() != myID)
		{
			if(tempplayer->getDirection() == 0)
			{
				int randomdir = rand()%5;
				if(randomdir !=0)
				{
					//cout<<"random turn"<<endl;
					//give them a random direction and reset vars
					tempplayer->setIntended(randomdir);
				}
			} 
		}




		//first off, if you're a ghost, check your death counter to see if you need
		//to be brought back
		if(tempplayer->isAGhost())
		{
			Ghost * tempghost = (Ghost*)tempplayer;
			//if they're dead, check their deadcounter
			if(tempghost->isDead())
			{
				//try decrementing their death counter
				if(tempghost->decDeadCount())
				{
					tempghost->resetDead();
					//cout<<"reset dead in server loop"<<endl;
				}
			}
			//if they're not dead and not edible
			else if(!tempghost->isEdible())
			{
				//give them some points
				tempghost->changeScore(25);
			}
		}

		//if you're a pacman, check to see if you're dead
		if(!tempplayer->isAGhost())
		{
			Pacman * temppacman = (Pacman*)tempplayer;

			//if they're dead, check their deadcounter
			if(temppacman->isDead())
			{
				//try decrementing their death counter
				if(temppacman->decDeadCount())
				{
					//cout<<"dead count returned true"<<endl;
				}
				else
				{
					//else, make the pacman hold still
					temppacman->setDir(0);
					temppacman->setIntended(0);			
				}
			}
			
		}


		int idir = tempplayer->getIntended();
		int dir = tempplayer->getDirection();

		//if(tempplayer->getPlayerID() == 2)
		//cout<<"1 dir and idir "<<dir<<", "<<idir<<endl;

		Space * tempspace = tempplayer->getSpace();

		//check to see if they are at an intersection first.
		//that is, if their x and y coords are divisible by 6
		if( ((int)tempplayer->getX())%6 == 0 && ((int)tempplayer->getY())%6 == 0)
		{


			//they are at an intersection

	
			//temp stuff for emulating other players
			//they are at an intersection, pick a random way to go 30% of the time
			if(tempplayer->getPlayerID() != myID)
			{
				if(rand()%100 < 30)
				{
					int randomdir = rand()%7;
					if(randomdir !=0)
					{
						//cout<<"random intersection turn"<<endl;
						//give them a random direction and reset vars
						tempplayer->setIntended(randomdir);
						idir = tempplayer->getIntended();
						dir = tempplayer->getDirection();
					}
				}
			}



			//the player is already in the middle of the space
			//so they get the item, look to see what the item 
			//is
			bool isghost = tempplayer->isAGhost();
			char item = tempspace->getItemKey();


			//now, since they are in an intersection
			//check for any stuff in the space, and eat it
			//this does not include other players
			if(item == '.' && !isghost)
			{
				//cout<<"found a pill"<<endl;
				//item is a pill, pacman should eat it

				//eat the item by changing his score
				tempplayer->changeScore(100);
				//changeScore increments his score

				//remove the item
				tempspace->removeItem();
			}
			else if(item == '*' && !isghost)
			{
				//cout<<"found a power pill"<<endl;
				//item is a power pill, pacman eats it
				//eat the item by changing all of the
				//ghosts to Edible with SetEdible
				//this should actually be a function in
				//playerlist...I'll do that
				playerlist.setGhostsEdible();
				
				//there needs to be a timer that resets
				//the ghosts' edible bool
				edible = true;
				ediblecount = 0;

				//remove the item
				tempspace->removeItem();
			}

			//if they want to go the same way, throw a flag
			bool sameway = false;
			if(idir == dir)
				sameway = true;

			//if their current direction hits a wall
			bool hitsWall = true;
			char dirC = 'w';
			switch(dir){
				case 2:
					dirC = 'n';
					break;
				case 3:
					dirC = 'e';
					break;
				case 4:
					dirC = 's';
					break;
				case 5:
					dirC = 'u';
					break;
				case 6:
					dirC = 'd';
					break;
			}
			if(tempspace->getNeighbor(dirC)){
				hitsWall = false;
			}

			//and see if there is a neighbor in that direction
			//change dir
			switch(idir)
			{
			case 1:
				if(tempspace->getNeighbor('w'))
				{
					tempplayer->setDir(idir);
				}
				else
				{
					if(sameway || hitsWall)
					{
						//make them stop because they are running into a wall
						tempplayer->setDir(0);
						tempplayer->setIntended(0);
					}

				}
		
				break;
			case 2:
				if(tempspace->getNeighbor('n'))
				{
					tempplayer->setDir(idir);
					//cout<<"yup, there's an north neighbor"<<endl;
				}
				else
				{
					if(sameway || hitsWall)
					{
						//make them stop because they are running into a wall
						tempplayer->setDir(0);
						tempplayer->setIntended(0);
					}
				}
				break;
			case 3:
				if(tempspace->getNeighbor('e'))
				{
					tempplayer->setDir(idir);
					//cout<<"yup, there's a right neighbor"<<endl;
				}
				else
				{
					//cout<<"nope no right neighbor"<<endl;
					if(sameway || hitsWall)
					{
						//make them stop because they are running into a wall
						tempplayer->setDir(0);
						tempplayer->setIntended(0);
					}

				}
				break;
			case 4:
				if(tempspace->getNeighbor('s'))
				{
					tempplayer->setDir(idir);
					//cout<<"yup, there's a south neighbor"<<endl;
				}
				else
				{
					if(sameway || hitsWall)
					{
						//make them stop because they are running into a wall
						tempplayer->setDir(0);
						tempplayer->setIntended(0);
					}
				}
				break;
			case 5:
				if(tempspace->getNeighbor('u'))
				{
					tempplayer->setDir(idir);
					//cout<<"yup, there's an up neighbor"<<endl;
				}
				else
				{
					if(sameway || hitsWall)
					{
						//make them stop because they are running into a wall
						tempplayer->setDir(0);
						tempplayer->setIntended(0);
					}
				}
				break;
			case 6:
				if(tempspace->getNeighbor('d'))
				{
					tempplayer->setDir(idir);
					//cout<<"yup, there's a down neighbor"<<endl;
				}
				else
				{
					if(sameway || hitsWall)
					{
						//make them stop because they are running into a wall
						tempplayer->setDir(0);
						tempplayer->setIntended(0);
					}
				}
				break;
			default:
				break;
			}
	

			//finally reset dir and idir so we know where we're going
			idir = tempplayer->getIntended();
			dir = tempplayer->getDirection();

			//if(tempplayer->getPlayerID() == 2)
			//cout<<"dir and idir before collisions "<<dir<<", "<<idir<<endl;


		}
		//else if they are not at an intersection
		else
		{

			//if they aren't at an intersection, they're probably reversing directions
			//or starting again after a collision, so just check to see which coord
			//is not divisible by 6
			if((idir == 1) || (idir == 3))
			{
				//if they're between two horizontal spaces, let them change
				if( ((int)tempplayer->getX())%6 != 0 ) 
				{
					tempplayer->setDir(idir);
					dir = idir;
				}
			}
			else if( (idir == 2 || idir == 4))
			{
				//if they're between two vertical spaces, let them change
				if( ((int)tempplayer->getY())%6 != 0 ) 
				{
					tempplayer->setDir(idir);
					dir = idir;
				}
				
			}

		}

		//check for collisions here...this is where pacmen eat ghosts and vice versa
	        map <int, Player*>::iterator j;

		//loop through all the other players
		for(j = tempplayermap->begin(); j != tempplayermap->end(); j++)
		{
			
			Player * otherguy = j->second;
			int thisID = tempplayer->getPlayerID();
			int hisID = otherguy->getPlayerID();

			//don't compare against yourself, or with guys on different levels
			if(otherguy != tempplayer && playerlist.onSameZLevel(thisID, hisID))
			{

				GLfloat xsep = playerlist.getXSep(thisID, hisID);
				GLfloat ysep = playerlist.getYSep(thisID, hisID);
	
				//if(tempplayer->getPlayerID() == 2)
				//cout<<"thisID, hisID, xsep, ysep: "<<thisID<<", "<<hisID<<", "<<xsep<<", "<<ysep<<endl;

				//if(tempplayer->getPlayerID() == 2)
				//cout<<"3 dir and idir "<<dir<<", "<<idir<<endl;

				switch(dir)
				{
				case 1: //you're moving left
					if(xsep <= 3 && xsep > 0 &&  ysep == 0)
					{
						//collision, do something
						//cout<<"this ID: "<<thisID<<endl;

						//cout<<"collision dir 1 idir:"<<idir<<endl;
						//if they are both pacmen, stop this one...this could be booled out
						if(!tempplayer->isAGhost() && !otherguy->isAGhost())
						{
							//stop myself here
							tempplayer->setDir(0);
							dir = 0;

							//but let him go the other way
							if(idir == 3)
							{
								//cout<<"direction set for 3"<<endl;
								tempplayer->setIntended(3);
								tempplayer->setDir(3);
								dir = 3;
							}
						}
						Collision(tempplayer, otherguy);
						
					}
					break;
				case 2:
					if(ysep >= -3 && ysep < 0 &&  xsep == 0)
					{
						//collision, do something
						//cout<<"collision"<<endl;
						
						//if they are both pacmen, stop them...this could be booled out
						if(!tempplayer->isAGhost() && !otherguy->isAGhost())
						{
							//stop myself here
							tempplayer->setDir(0);
							dir = 0;

							//but let them go the other way
							if(idir == 4)
							{
								tempplayer->setIntended(4);
								tempplayer->setDir(4);
								dir = 4;
							}
						}
						Collision(tempplayer, otherguy);
					}
					break;
				case 3:	
					if(xsep >= -3 && xsep < 0 &&  ysep == 0)
					{
						//collision, do something
						//cout<<"collision"<<endl;
						
						//if they are both pacmen, stop them...this could be booled out
						if(!tempplayer->isAGhost() && !otherguy->isAGhost())
						{
							//stop myself here
							tempplayer->setDir(0);
							dir = 0;

							//but let them go the other way
							if(idir == 1)
							{
								tempplayer->setIntended(1);
								tempplayer->setDir(1);
								dir = 1;
							}
						}
						Collision(tempplayer, otherguy);
					}
					break;
				case 4:
					if(ysep <= 5 && ysep > 0 &&  xsep == 0)
					{
						//collision, do something
						//cout<<"collision"<<endl;
						
						//if they are both pacmen, stop them...this could be booled out
						if(!tempplayer->isAGhost() && !otherguy->isAGhost())
						{
							//stop myself here
							tempplayer->setDir(0);
							dir = 0;

							//but let them go the other way
							if(idir == 2)
							{
								tempplayer->setIntended(2);
								tempplayer->setDir(2);
								dir = 2;
							}
						}
						Collision(tempplayer, otherguy);
					}

					break;
				default:
					break;
				}
			}
		}		
		





		//move the player here
		GLfloat move = 3;

		//if they're an edible and not dead ghost, use a different movement unit
		//make a temp ghost*
		if(tempplayer->isAGhost())
		{
			Ghost * tempghost = (Ghost*)tempplayer;

			//if the ghost is not dead
			if(tempghost->isEdible() && !tempghost->isDead())
			{
				move = 1.5;
			}
		}
		

		//cout<<"about to move player "<<tempplayer->getPlayerID()<<" in direction: "<<dir<<endl;
		switch(dir)
		{
		case 1:
			tempplayer->SetX(tempplayer->getX()-move);
			break;
		case 2:
			tempplayer->SetY(tempplayer->getY()+move);
			break;
		case 3:
			tempplayer->SetX(tempplayer->getX()+move);
			break;
		case 4:
			tempplayer->SetY(tempplayer->getY()-move);
			break;
		case 5:
			tempplayer->SetZ(tempplayer->getZ()+1);
			tempplayer->setIntended(1);
			tempplayer->setDir(0);

			//reset space UP!!
			tempplayer->changePosition(tempspace->getNeighbor('u'));
			break;
		case 6:
			tempplayer->SetZ(tempplayer->getZ()-1);
			tempplayer->setIntended(1);
			tempplayer->setDir(0);

			//reset space DOWN!!
			tempplayer->changePosition(tempspace->getNeighbor('d'));
			break;
		default:
			break;
		}

		//if the player's X and Y coords are greater than 6 units away from the space that they should be
		//occupying, put them in the new space and leave their coords alone
		GLfloat spacex = tempspace->getX()*6;
		GLfloat spacey = tempspace->getY()*-6;
		GLfloat x = tempplayer->getX();
		GLfloat y = tempplayer->getY();

		GLfloat xdiff = spacex - x;
		GLfloat ydiff = spacey - y;

		if(xdiff >= 6 || xdiff <=-6 || ydiff >= 6 || ydiff <=-6)
		{
			//set player's Space * to a neighboring pointer
			//cout<<"x and y diffs: "<<xdiff<<", "<<ydiff<<endl;
			if(xdiff >= 6)
			{
				tempplayer->changePosition(tempspace->getNeighbor('w'));
				//cout<<"reset space west"<<endl;
			}
			else if(xdiff <= -6)
			{
				tempplayer->changePosition(tempspace->getNeighbor('e'));
				//cout<<"reset space east"<<endl;
			}
			else if(ydiff >= 6)
			{
				tempplayer->changePosition(tempspace->getNeighbor('s'));
				//cout<<"reset space south"<<endl;
			}
			else if(ydiff <= -6)
			{
				tempplayer->changePosition(tempspace->getNeighbor('n'));
				//cout<<"reset space north"<<endl;
			}
			

			//after the Space * has been set, set the coords of pacman equal to his new space
			Space * newspace = tempplayer->getSpace();
			tempplayer->SetX( newspace->getX()*6);
			tempplayer->SetY( newspace->getY()*-6);
			
		}
	}
	
	delete tempplayermap;
}

//OpenGL setup function, I don't think the SMOOTH Enables actually do anything
//but who am I to argue
void SetupRC()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);

	//glEnable(GL_DEPTH_TEST);

	//these were killing the performance on UGCS
	//glEnable(GL_POINT_SMOOTH);
	//glEnable(GL_LINE_SMOOTH);
	//glEnable(GL_POLYGON_SMOOTH);

}

//handles the window function
//remember to make it full size in the final version
void Reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//a very interesting function, much easier than doing aspect ratios by hand
//	gluPerspective(50.0, (GLfloat)w/(GLfloat)h, 1.0, 800.0);

	//took perspective out because this is faster...

	if(w<=h)
		glOrtho(-50.0, 50.0, -50.0*h/w, 50.0*h/w, -1.0, 1.0);
	else
		glOrtho(-50.0*w/h, 50.0*w/h, -50.0, 50.0, -1.0, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}




//this is a pretty lame key catching process as described by one of the groups
//in the last presentations, but it make be perfect for what we're doing, because
//the length of time that you hold the key down doesn't matter in pacman
//this works using keyboard messages, which are generated once, and then are generated
//continuously after a certain time has elapsed.
//if a different key is pressed, one new message is sent immediately.
//this may be a problem however if it doesn't account for multiple keys at a time
//but I don't know how to get the SHORT AsyncKeyState() in linux....
void SpecialKeyEvent(int key, int x, int y)
{

	#ifndef SINGLEPLAYER
	// 'k' (long)timestamp (byte)id (glenum)GLUT_KEY_*
	byte packet_data[256];
	int pos = 0;
	*(packet_data+pos) = 'k'; pos++;
	*((long*)(packet_data+pos)) = ms; pos+=sizeof(long);
	*(packet_data+pos) = (byte)myID; pos++;
	*((GLenum*)(packet_data+pos)) = key; pos+=sizeof(GLenum);
	outpacket->setData(packet_data,pos);
	//cerr << "send : " << outpacket->getLength() << " " << (char)(outpacket->getData()[0]) << "\n";
	rudp->sendReliable(outpacket);
	#endif

	#ifdef SINGLEPLAYER
	switch(key)
	{
	case GLUT_KEY_DOWN:
		playerlist.getPlayer(myID)->setIntended(4);
		break;
	case GLUT_KEY_RIGHT:
		playerlist.getPlayer(myID)->setIntended(3);
		break;
	case GLUT_KEY_UP:
		playerlist.getPlayer(myID)->setIntended(2);
		break;
	case GLUT_KEY_LEFT:
		playerlist.getPlayer(myID)->setIntended(1);
		break;
	case GLUT_KEY_PAGE_UP:
		playerlist.getPlayer(myID)->setIntended(5);
		break;
	case GLUT_KEY_PAGE_DOWN:
		playerlist.getPlayer(myID)->setIntended(6);
		break;

	default:
		break;
	}
#endif

	//very important to PostRedisplay after a change in the graphics that's outside the main thread.
	glutPostRedisplay();
}

//this is a pretty lame key catching process as described by one of the groups
//in the last presentations, but it make be perfect for what we're doing, because
//the length of time that you hold the key down doesn't matter in pacman
//this works using keyboard messages, which are generated once, and then are generated
//continuously after a certain time has elapsed.
//if a different key is pressed, one new message is sent immediately.
//this may be a problem however if it doesn't account for multiple keys at a time
//but I don't know how to get the SHORT AsyncKeyState() in linux....
//key even is only used for chat too.....and zooming and music....
void KeyEvent(unsigned char key, int x, int y)
{
	string message;

	if(glutGetModifiers() != GLUT_ACTIVE_CTRL)
	{
		switch(key)
		{
		//the backspace key gumshoe
		case 0x08:  
			chatwnd.deleteChar();
			break;
		//the enter key
		case 0x0d:
			message = chatwnd.getString();
			//send message using the network here
				
			message = "c" + playerlist.getPlayer(myID)->getName() + ":" + message;
			if(tcp!=NULL)
			{
				tcp->sendString(&message);
			}
			break;
	
		//all key inputs should go to the chat window...
		default:
			//this will have to be changed to support GameBoard
			chatwnd.addChar(key);
		}
	}

	//this function is really pretty sweet....it's glut's little way
	//of catching ctrl, shift and alt, it took my forever to find!!!
	//******important note:
	//for some reason, a is 1 when it's attached to ctrl, and 97 when 
	//ctrl is not held down....I have no idea why!
	if(glutGetModifiers() == GLUT_ACTIVE_CTRL && key == 1)
	{
		//time to zoom out
		zoomlevel*=.95;
	}
	if(glutGetModifiers() == GLUT_ACTIVE_CTRL && key == 26)
	{
		//time to zoom in
		if(zoomlevel<.5)
		{
			zoomlevel*=1.05;
		}
	}

	glutPostRedisplay();
}

//use this function to set up OpenGL
void OpenGL::InitializeOpenGL()
{
//	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
#ifndef __APPLE__
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowSize(700, 700);
#endif
#ifdef __APPLE__
	glutInit(NULL, 0);										
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowPosition(100, 100);
#endif	
	glutCreateWindow("MIP");

	SetupRC();

	//this needs to be changed to use the glTimeDelayFunc() declaration.
	//I think it takes 2 parameters.  a void * for the function to call
	//and an int for the amount of time to wait before calling it
	//it'll work cross platform
	glutDisplayFunc(RenderScene);
	glutReshapeFunc(Reshape);

	//declaration of keyboard handlers..
	//may need to be changed, see above explanations
	glutKeyboardFunc(KeyEvent);
	glutSpecialFunc(SpecialKeyEvent);
}

void OpenGL::StartMainLoop()
{
	//this needs to be changed to use the glTimeDelayFunc() declaration.
	//I think it takes 2 parameters.  a void * for the function to call
	//and an int for the amount of time to wait before calling it
	//it'll work cross platform
	glutMainLoop();
}


//function of the OpenGL class that adds the texture into memory using the
//glBindTexture function.
//this method has an opposite removeBitmap method, but it is not implemented yet
GLuint OpenGL::AddBitmap(BITMAPINFO *TexInfo, GLubyte *TexBits, bool blackclear)
{
	//this will hold the texture ID
	//we want to make one texture at a time
	GLuint newtexture[1];
	newtexture[0] = 42;
	glGenTextures(1, &newtexture[0]); //generate the texture's ID

	//if this is the first one, we need to allocate some space for it
	if(numtextures == 0)
	{
		texture_objects=(GLuint*)malloc(sizeof(GLuint));
		texture_objects[0]=newtexture[0];
	}

	//BindTexture is an OpenGL API function, and it's very confusing :/
	//too much so to go into detail here.
    glBindTexture(GL_TEXTURE_2D, newtexture[0]);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	//if the black is clear then the channel has an alpha layer
	//if we add support for pictures with alpha channels already then this variable should
	//be changed to alphaexists or something like that because this is misleading
	if(blackclear)
	{
		glTexImage2D (GL_TEXTURE_2D, 0, 4, TexInfo->bmiHeader.biWidth, TexInfo->bmiHeader.biHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, TexBits);
	}
	//if the black isn't cleared, then the bmp has no alpha channel
	//again, this variable needs to be changed if we support images that inheirently
	//have an alpha channel
	else
	{
		glTexImage2D (GL_TEXTURE_2D, 0, 3, TexInfo->bmiHeader.biWidth, TexInfo->bmiHeader.biHeight, 0, GL_BGR_EXT, GL_UNSIGNED_BYTE, TexBits);
	}

	return newtexture[0];
}

/*
//initialise the ghost and pacman sprites for animation
//we will probably also need a Ms Pacman sprite because of her lips :/
//this must only be called AFTER OpenGL has initialized.
//OpenGL is initialized in the constructor
bool InitializeSprites(void)
{
	for(int i=0; i<10; i++)
	{
		char * filename;
		filename=(char*)malloc(sizeof(char)*256);

		sprintf(filename, "pacman%i.bmp", i);

		PACMAN_SPRITE[i] = Image::addImage(filename, true);

		free(filename);
	}

	GHOST_SPRITE[0] = Image::addImage("ghost0.bmp", true);
	GHOST_SPRITE[1] = Image::addImage("ghost1.bmp", true);

	return 1;
}
*/

//simple constructor.
OpenGL::OpenGL(void)
{
	InitializeOpenGL();
	//can't use sprites so liberally anymore
//	InitializeSprites();
}
