#include "FileException.h"
#include <iostream>
#include <string>
#include <vector>
#include "image.h"
#include "opengl.h"
#include "playerlist.h"
#include "pacskin.h"
#include "TCP.h"
#include "RUDP.h"
#include "chatwnd.h"
#include "Level.h"
#include "Space.h"

//simple test program.  It's important that the objects be initialized in this order
//because some of the initialzation steps that player list goes through
//won't work it opengl itself has not been initialized yet
OpenGL opengl;
PlayerList playerlist;
int myID;
TCP* tcp = NULL;
RUDP* rudp = NULL;
ChatWnd chatwnd;
Level *testlevel;
long ms;
UDPacket* outpacket;
pthread_mutex_t graphicsMutex;

//this function is what we use for single player mode.  Right now, the single
//and multiplayer modes are two separate executables, the usage for the single player
//mode is
// isAGhost, numfloors, numpacmen, numghosts
//note, there will be at least one of each, pacmen and ghost
int main(int argc, char* argv[])
{
	if(argc != 5)
	{
		cout<<"Usage " << argv[0] << " is_a_ghost num_floors num_pacmen num_ghosts" 
				<<endl;
		cout<<"  is_a_ghost is 0 for being a Pacman and 1 for being a Ghost"<<endl;
		exit(0);
	}
	
	//initialization vars
	bool isghost = atoi(argv[1]);
	int floors = atoi(argv[2]);
	int numpacs = atoi(argv[3]);
	int numghosts = atoi(argv[4]);



	//test skin object.  It's easiest to make these and then pass them into the addplayer function
	pthread_mutex_init(&graphicsMutex,NULL);

	//create the new level, unfortunately this is global
	try{
	  testlevel = new Level(50, 40, floors, numpacs, numghosts);
	}
	catch(FileException f){
	  cerr<<"Level failed:"<<endl;
	  f.print(cerr);
	  exit(1);
	}
cerr<<"ok"<<endl;
	//create pacman with a skin, this skin is blue dots
	PacSkin tex("pacskin.bmp");
	int pac = playerlist.addplayer("player 0", 0, &tex);
	//int pac = playerlist.addplayer("player 0", 0, NULL);


	//create all the pacmen
	for(int i = 1; i<numpacs; i++)
	{
		char name[16];
		sprintf(name, "player %i",i) ;

		//add them to the playerlist
		playerlist.addplayer(name, 0, NULL);
	}

	//create at least one ghost in case that's what they wanted to be
	int ghost = playerlist.addplayer("ghost 0", 0, NULL, 1);

	//create the ghosts
	for(int i = 1; i<numghosts; i++)
	{
		char name[16];
		sprintf(name, "ghost %i",i) ;

		//add them to the player list
		playerlist.addplayer(name, 0, NULL, 1);
	}


	//set the user's ID, this is used in drawing
	if(!isghost)
		myID = pac;
	else
		myID = ghost;

	
	//loop through all the players and set them up within the level.
	//this should probably go in Player construction, but right now
	//everything is global, so it wouldn't have been much prettier
	//to extern Level inside the Player class
        map <int, Player*>::iterator i;
        map <int, Player*> * tempplayermap = playerlist.getPlayerMap();
        for(i = tempplayermap->begin(); i != tempplayermap->end(); i++)
        {                       
		//put him into a space
		Space * tempspace;
		Player * tempplayer = i->second;

		//assign appropriate space type
		if(tempplayer->isAGhost())
			tempspace = testlevel->getGhostStart();
		else
			tempspace = testlevel->getPacmanStart();

		//change the player's position, this function automatically sets the
		//player's coords too, very important
		tempplayer->changePosition(tempspace);

		//output for debugging
		cout<<"x y and z: "<<tempplayer->getX()<<", "<<tempplayer->getY()<<", "<<tempplayer->getZ()<<endl;
	}


	//start the opengl drawing functions
	opengl.StartMainLoop();

	return 0;
}











